@@ -2,7 +2,7 @@
 // GB_ew: ewise kernels for each built-in binary operator
 //------------------------------------------------------------------------------
 
-// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2024, All Rights Reserved.
+// SuiteSparse:GraphBLAS, Timothy A. Davis, (c) 2017-2025, All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
 
 //------------------------------------------------------------------------------
@@ -17,7 +17,6 @@
 #if GB_TYPE_ENABLED
 #include "GB.h"
 #include "emult/GB_emult.h"
-#include "slice/GB_ek_slice.h"
 #include "assign/GB_bitmap_assign_methods.h"
 #include "FactoryKernels/GB_ew__include.h"
 
@@ -44,6 +43,7 @@
 
 #define GB_CTYPE_IS_ATYPE 0
 #define GB_CTYPE_IS_BTYPE 0
+#define GB_Cp_IS_32 Cp_is_32
 
 // disable this operator and use the generic case if these conditions hold
 #if (defined(GxB_NO_NE) || defined(GxB_NO_INT64) || defined(GxB_NO_NE_INT64))
@@ -225,7 +225,7 @@ GrB_Info GB (_AemultB_08__ne_int64)
     #if GB_DISABLE
     return (GrB_NO_VALUE) ;
     #else
-    #include "emult/template/GB_emult_08_meta.c"
+    #include "emult/template/GB_emult_08_template.c"
     return (GrB_SUCCESS) ;
     #endif
 }
@@ -242,7 +242,7 @@ GrB_Info GB (_AemultB_02__ne_int64)
     const bool Mask_comp,
     const GrB_Matrix A,
     const GrB_Matrix B,
-    const int64_t *restrict Cp_kfirst,
+    const uint64_t *restrict Cp_kfirst,
     const int64_t *A_ek_slicing,
     const int A_ntasks,
     const int A_nthreads
@@ -267,7 +267,7 @@ GrB_Info GB (_AemultB_04__ne_int64)
     const bool Mask_struct,
     const GrB_Matrix A,
     const GrB_Matrix B,
-    const int64_t *restrict Cp_kfirst,
+    const uint64_t *restrict Cp_kfirst,
     const int64_t *M_ek_slicing,
     const int M_ntasks,
     const int M_nthreads
@@ -369,7 +369,7 @@ GrB_Info GB (_bind1st_tran__ne_int64)
     GrB_Matrix C,
     const GB_void *x_input,
     const GrB_Matrix A,
-    int64_t *restrict *Workspaces,
+    void **Workspaces,
     const int64_t *restrict A_slice,
     int nworkspaces,
     int nthreads
@@ -380,6 +380,7 @@ GrB_Info GB (_bind1st_tran__ne_int64)
     return (GrB_NO_VALUE) ;
     #else
     GB_X_TYPE x = (*((const GB_X_TYPE *) x_input)) ;
+    bool Cp_is_32 = C->p_is_32 ;
     #include "transpose/template/GB_transpose_template.c"
     return (GrB_SUCCESS) ;
     #endif
@@ -404,7 +405,7 @@ GrB_Info GB (_bind2nd_tran__ne_int64)
     GrB_Matrix C,
     const GrB_Matrix A,
     const GB_void *y_input,
-    int64_t *restrict *Workspaces,
+    void **Workspaces,
     const int64_t *restrict A_slice,
     int nworkspaces,
     int nthreads
@@ -414,10 +415,13 @@ GrB_Info GB (_bind2nd_tran__ne_int64)
     return (GrB_NO_VALUE) ;
     #else
     GB_Y_TYPE y = (*((const GB_Y_TYPE *) y_input)) ;
+    bool Cp_is_32 = C->p_is_32 ;
     #include "transpose/template/GB_transpose_template.c"
     return (GrB_SUCCESS) ;
     #endif
 }
 
+#else
+GB_EMPTY_PLACEHOLDER
 #endif
 
